#!/etc/opt/openit/perl -w
#
# Open iT Inc. Philippines
#
# Description:
# 	Fixes licpoll2 archive Max In Use > Max Available
# 	due to polled check-out package without check-in.
# 	Adds check-in record for packages, based on check-in
# 	records of a component (by a user), belonging to the package.
#
# Reference: RM#7831
#
# Author: Ivan Bilon (ibilon@openit.com)
#


BEGIN {
	if ( $^O eq 'MSWin32' ) {
		push @INC, "../lib";
	}
	else {
    	push @INC, "/opt/openit/contrib/lib";
	}  
}

use strict;
use if $^O eq 'MSWin32' , "OpeniT::Config";
use OpeniT::Contrib::Config;
use File::Basename;
use Cwd;
use Env qw(@PATH) ;

# --------------------- HELPER FUNCTIONS ------------------------

sub testdir {
	my ($path, $what) = @_;
	if(! -d $path) {
		die "$what directory does not exist: $path";
	}
}

sub testfile {
	my ($path, $what) = @_;
	if(! -e $path) {
		die "$what directory does not exist: $path";
	}
}

sub cleanpath {
	my ($path) = @_;
	$path =~ tr{\\}{/};
	return $path;
}

sub getdirsnumeric {
	my ($path) = @_;
	my @list;
	opendir(DIR, $path) or die $!;
	while(my $dir = readdir(DIR)) {
		next unless (-d $dir);
		next unless ($dir =~ /\d+/);
		push @list, $dir;
	}
	closedir(DIR);
	return sort @list;
}

sub traversearchive {
	my ($dir, $from, $to, $handler) = @_;
	my ($s_year, $s_month, $s_day) = split '-', $from;
	my ($e_year, $e_month, $e_day) = split '-', $to;
	chdir $dir;
	my @years = getdirsnumeric(".");
	foreach my $year (@years) {
		next unless ($year >= $s_year && $year <= $e_year);
		my $start_month = ($year == $s_year) ? $s_month : 1;
		my $end_month = ($year == $e_year) ? $e_month : 12;
		chdir $year;
		my @months = getdirsnumeric(".");
		foreach my $month (@months) {
			next unless ($month >= $start_month && $month <= $end_month);
			my $start_day = ($year == $s_year && $month == $s_month) ? $s_day : 1;
			my $end_day = ($year == $e_year && $month == $e_month) ? $e_day : 31;
			chdir $month;
			my @days = getdirsnumeric(".");
			foreach my $day (@days) {
				next unless ($day >= $start_day && $day <= $end_day);
				chdir $day;
				$handler->($year, $month, $day);
				chdir "../";
			}
			chdir "../";
		}
		chdir "../";
	}
}

# ------------------------- CONFIG ----------------------------

my $config = new OpeniT::Contrib::Config;
if( $^O eq 'MSWin32' ) {
	$config->read(&OpeniT::Config::CONFIG_DIR . "/openit.cfg");    # Additional config
} else {
	$config->read("/etc/opt/openit/openit_contrib.cfg");    # Additional config
}

my $ROOT_DIR 		= cleanpath $config->{ROOT_DIR};
my $BIN_DIR 		= cleanpath $config->{BIN_DIR};
my $CONFIG_DIR 		= cleanpath $config->{CONFIG_DIR};
my $ROOT_DATA_DIR 	= cleanpath $config->{ROOT_DATA_DIR};
my $ARCHIVE_DIR 	= "$ROOT_DATA_DIR/archives/license";

testdir $ROOT_DIR, "Open iT Root";
testdir $BIN_DIR, "Open iT Binary";
testdir $CONFIG_DIR, "Open iT Configuration";
testdir $ROOT_DATA_DIR, "Open iT Database";
testdir $ARCHIVE_DIR, "Archive";

# ------------------------- MAIN ----------------------------

sub DEFAULTMSG {
	print "Invalid arguments. Please see --help for more details.\n";
	exit 0;
}

sub PRINTHELP {
	print "Fixes Open iT licpoll2 Archive for Packages with No Check-ins\n";
	print "This program adds check-ins on Packages when its Component gets checked-in.\n";
	print "This should fix the Max In Use > Max Available bug in the report which is caused by licpoll.\n";
	print "\n";
	print "Arguments: <start year>-<start-month> <end year>-<end-month>\n";
	print "Example: 2016-1 2016-3\n";
	print "    This means to fix licpoll2 archives from January to February 2016.\n";
	exit 0;
}

if (!defined $ARGV[0]) {
	DEFAULTMSG();
}

if ($ARGV[0] eq "--help") {
	PRINTHELP();
}

my ($START_YEAR, $START_MONTH) 		= split '-', $ARGV[0];
my ($END_YEAR, $END_MONTH) 			= split '-', $ARGV[1];

if (!$START_YEAR || !$START_MONTH || !$END_YEAR || !$END_MONTH) {
	DEFAULTMSG();
}

my $CONTRIB_BIN_DIR = "$ROOT_DIR/contrib/bin";
testdir $CONTRIB_BIN_DIR, "Open iT Contrib Bin";

my $AWKFILE = "$CONTRIB_BIN_DIR/fix_licpoll2_package_checkin.awk";
my $AWKFILE2 = "$CONTRIB_BIN_DIR/fix_licpoll2_feature_checkin.awk";
my $AWK = "awk";

my $LICPOLL2_DIR = "$ARCHIVE_DIR/licpoll2";
testdir $LICPOLL2_DIR, "licpoll2";

chdir $LICPOLL2_DIR;

sub handler {
	my ($year, $month, $day) = @_;
	opendir my $dir, '.' or die $!;
	open CMD, "| gunzip *.in.gz" or die $!;
	close CMD;
	while(my $file = readdir($dir)) {
		next unless (-f $file);
		next unless ($file =~ /.in$/);
		my $in 		= $file;
		my $tmp  	= "$in.tmp";
		open CMD, "| $AWK -f \"$AWKFILE\" \"$in\" | \"$AWK\" -f \"$AWKFILE2\" > \"$tmp\"" or die $!;
		close CMD;
		rename $tmp, $in;
		unlink $tmp;
	}
	open CMD, "| gzip *.in" or die $!;
	close CMD;
	closedir $dir;
}

traversearchive(".", "$START_YEAR-$START_MONTH-1", "$END_YEAR-$END_MONTH-31", \&handler);
